import os
import shutil

import click

from lucy import update_snippets as us, utils
from lucy.tester import Tester
from lucy.config import Config, Website
from lucy.parser_.contest import Contest


@click.group()
def cli() -> None:
    pass


@cli.command('update-snippets')
@click.option('-ed',
              '--entry-dir',
              'entry_dir_',
              default=Config.COMMONS_PATH,
              type=click.Path(exists=True))
@click.option('-o', '--out', 'out', default=Config.SNIPPETS_PATH, type=click.Path())
def update_snippets(entry_dir_: str, out: str) -> None:
    entry_dir_ = os.path.abspath(entry_dir_)
    out = os.path.abspath(out)
    snippets = us.run(entry_dir_, out)
    click.secho(f'Found {len(snippets)} snippets.', fg='green', bold=True)
    for snippet in snippets:
        click.echo(snippet)


@cli.command('setup')
@click.argument('site', type=Config.CLI_WEBSITE_CHOICE)
@click.argument('contest_id')
def setup(site: str, contest_id: str) -> None:
    website = Website.from_string(site)
    contest_ = Contest(website, contest_id)
    for task in contest_.parser.tasks:
        impl_path = utils.get_impl_path(website, contest_id, task.id_)
        if not os.path.exists(impl_path):
            shutil.copy(Config.TEMPLATE_PATH, impl_path)


@cli.command('test')
@click.argument('site', type=Config.CLI_WEBSITE_CHOICE)
@click.argument('contest_id')
@click.argument('task_id')
def test(site: str, contest_id: str, task_id: str) -> None:
    website = Website.from_string(site)
    tester = Tester(website, contest_id, task_id)
    tester.run()



from __future__ import annotations
import os
import subprocess
from typing import Generator, Tuple

import click

from lucy import utils
from lucy.config import SampleType, Website


# pylint: disable=too-few-public-methods
class Tester:

    def __init__(self, website: Website, contest_id: str, task_id: str) -> None:
        self.website = website
        self.contest_id = contest_id
        self.task_id = task_id

        self.num_tests = len(
            os.listdir(utils.get_sample_path(website, contest_id, task_id, SampleType.IN)))

        self.impl_path = utils.get_impl_path(website, contest_id, task_id)
        self.bin_path = utils.get_impl_path(website, contest_id, task_id, bin_=True)

    def __compile(self) -> None:
        subprocess.check_call(['g++', self.impl_path, '-o', self.bin_path])

    @staticmethod
    def __read(path: str) -> str:
        with open(path) as f:  # pylint: disable=unspecified-encoding
            return f.read()

    def __tests(self) -> Generator[Tuple[str, str], None, None]:
        for idx in range(self.num_tests):
            in_path = utils.get_sample_path(self.website, self.contest_id, self.task_id,
                                            SampleType.IN, idx)
            out_path = utils.get_sample_path(self.website, self.contest_id, self.task_id,
                                             SampleType.OUT, idx)
            yield (Tester.__read(in_path), Tester.__read(out_path))

    def __exec(self, in_txt: str, truth_txt: str) -> bool:
        with subprocess.Popen([self.bin_path], stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE) as process:
            assert process.stdin
            process.stdin.write(in_txt.encode())
            process.stdin.close()
            assert process.stdout
            out_txt = process.stdout.read().decode()
            process.wait()
        return out_txt.strip() == truth_txt.strip()

    def run(self) -> None:
        self.__compile()

        for i, (in_txt, out_txt) in enumerate(self.__tests()):
            click.echo(f'Test#{i:02d}', nl=False)
            click.echo(f'{"." * 50}', nl=False)
            passes = self.__exec(in_txt, out_txt)
            if passes:
                click.secho('AC', bg='green')
            else:
                click.secho('WA', bg='red')



from __future__ import annotations
from dataclasses import dataclass
from enum import Enum
import os

import click


class Website(Enum):
    ATCODER = 1

    @staticmethod
    def from_string(website: str) -> Website:
        return Website[website.upper()]

    def __str__(self) -> str:
        if self == Website.ATCODER:
            return 'AtCoder'
        raise NotImplementedError()


class SampleType(Enum):
    IN = 1
    OUT = 2

    def __str__(self) -> str:
        return self.name.lower()


@dataclass
class Config:
    WEBSITE_HOST = {Website.ATCODER: 'https://atcoder.jp'}

    LUCY_HOME = os.getenv('LUCY_HOME') or f'{os.getenv("HOME")}/.lucy'
    LUCY_HOME = os.path.abspath(LUCY_HOME)
    os.makedirs(LUCY_HOME, exist_ok=True)

    SNIPPETS_DIR = f'{LUCY_HOME}/.vscode'
    os.makedirs(SNIPPETS_DIR, exist_ok=True)
    SNIPPETS_PATH = os.path.abspath(f'{SNIPPETS_DIR}/cp.code-snippets')

    COMMONS_PATH = os.path.abspath(f'{LUCY_HOME}/common')
    os.makedirs(COMMONS_PATH, exist_ok=True)

    TEMPLATE_PATH = f'{COMMONS_PATH}/base.cpp'
    with open(TEMPLATE_PATH, 'w+', encoding='utf-8'):
        pass

    CLI_WEBSITE_CHOICE = click.Choice(['atcoder'])

    IMPL_MAIN = 'main.cpp'
    IMPL_BIN = 'main'

    SAMPLES_DIR = 'tests'



import os
import shutil
from typing import Any

import click

from lucy import utils
from lucy.config import Config, SampleType, Website
from lucy.scraper import Scraper

from lucy.parser_.parser_ import Parser, Task


class AtCoderParser(Parser):

    def __parse_samples(self, task_id: str, task_page: Any) -> int:
        num_samples = 0

        samples_dir = utils.get_sample_path(Website.ATCODER, self.contest_id, task_id)
        if os.path.exists(samples_dir):
            shutil.rmtree(samples_dir)
        os.makedirs(utils.get_sample_path(Website.ATCODER, self.contest_id, task_id, SampleType.IN),
                    exist_ok=True)
        os.makedirs(utils.get_sample_path(Website.ATCODER, self.contest_id, task_id,
                                          SampleType.OUT),
                    exist_ok=True)

        for i, (input, output) in enumerate(utils.batched(task_page.select('pre[id]'), 2)):
            num_samples += 1

            in_path = utils.get_sample_path(Website.ATCODER, self.contest_id, task_id,
                                            SampleType.IN, i)
            with open(in_path, 'w+', encoding='utf-8') as f:
                f.write(input.text)

            out_path = utils.get_sample_path(Website.ATCODER, self.contest_id, task_id,
                                             SampleType.OUT, i)
            with open(out_path, 'w+', encoding='utf-8') as f:
                f.write(output.text)

        return num_samples

    def __parse_tasks(self, page: Any) -> list[Task]:
        tasks: list[Task] = []

        tasks_table = page.select('table.table tbody tr')
        for row in tasks_table:
            data = row.find_all('td')
            task_id, task_name = data[0].text, data[1].text

            task_path = f'{self.contest_id}_{task_id.lower()}'
            task_page = Scraper().get(f'{self.tasks_page_url}/{task_path}')
            num_samples = self.__parse_samples(task_id, task_page)
            click.secho(f'Found {num_samples} samples for task {task_id}.', fg='green', bold=True)

            task = Task(task_id, task_name)
            tasks.append(task)

        return tasks

    def __init__(self, contest_id: str) -> None:
        self.contest_id = contest_id

        scraper = Scraper()

        self.tasks_page_url = f'{Config.WEBSITE_HOST[Website.ATCODER]}/contests/{contest_id}/tasks'
        tasks_page = scraper.get(self.tasks_page_url)

        tasks = self.__parse_tasks(tasks_page)

        super().__init__(tasks)



from typing import Any, Generator, Optional

from lucy.config import Config, SampleType, Website


def batched(iterable: list[Any], n: int) -> Generator[tuple[Any, ...], None, None]:
    l = len(iterable)
    for ndx in range(0, l, n):
        yield tuple(iterable[ndx:min(ndx + n, l)])


def get_impl_path(website: Website,
                  contest_id: str,
                  task_id: str,
                  dir_: bool = False,
                  bin_: bool = False) -> str:
    impl_path = f'{Config.LUCY_HOME}/{website}/{contest_id.upper()}/{task_id.upper()}'
    if bin_:
        return f'{impl_path}/{Config.IMPL_BIN}'
    if not dir_:
        impl_path = f'{impl_path}/{Config.IMPL_MAIN}'
    return impl_path


def get_sample_path(website: Website,
                    contest_id: str,
                    task_id: str,
                    type_: Optional[SampleType] = None,
                    idx: Optional[int] = None) -> str:
    sample_path = get_impl_path(website, contest_id, task_id, dir_=True)
    sample_path += f'/{Config.SAMPLES_DIR}'
    if type_:
        sample_path += f'/{type_}'
        if idx is not None:
            sample_path += f'/{idx:02d}.txt'
    return sample_path



from lucy.config import Website

from lucy.parser_.atcoder import AtCoderParser


class Contest:

    def __init__(self, website: Website, contest_id: str) -> None:
        self.website = website

        match self.website:
            case Website.ATCODER:
                self.parser = AtCoderParser(contest_id.lower())
            case _:
                raise NotImplementedError()
